/*
 * Created on 18.05.2006
 *
 */
package big3d.listener;

import java.awt.Canvas;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import big3d.camera.Camera;
import big3d.utils.GLSettings;


public class CanvasListener implements MouseListener, MouseMotionListener,
MouseWheelListener, KeyListener  {
    
    /**
     * Variable zum Speichern des Canvas, auf das dieser Listener reagieren soll
     */
    private Canvas canvas;
    
    /**
     * Variable zum Speichern der Kamera, welche das Canvas beobachtet
     */
    private Camera cam;
    
    // fr die aktuelle Mausposition;
    /**
     * Variable zum Speichern der X-Ordinate des Mausklicks
     */
    private float mouseClickX;
    
    /**
     * Variable zum Speichern der Y-Ordinate des Mausklicks
     */
    private float mouseClickY;
    
    /**
     * Variable zum Speichern der X-Ordinate des Mauszeigers
     */
    private int mouseX;
 
    /**
     * Variable zum Speichern der Y-Ordinate des Mausklicks
     */
    private int mouseY;
    /**
     * Variable zum Speichern der Modelview-Matrix als 16-stelliges Array
     */
    
    double[] mvMatrix = new double[16];
    
    
    /**
     * Flag, ob die rechte Maustaste gedr&uuml;ckt wurde
     */
    private boolean rightMousePressed = false;
   
    /**
     * Flag, ob die 'strg'-Taste der Tastatur gedr&uuml;ckt wurde
     */
    private static boolean strgPressed = false;
 
    /**
     * Konstruktor.
     * 
     * <p>In diesem Konstruktor werden die verschiedenen ActionListener und die
     * Kamera angemeldet.</p>
     * 
     * @param c die Kamera
     */
    public CanvasListener(Camera c) {
        cam = c;  
        this.canvas = GLListener.canvas;
    }

    /**
     * Reagiert mit Kamerabewegungen auf Mausbewegungen mit
     * gedr&uuml;ckter Maustaste (und gegebenenfalls gedr&uuml;ckter 
     * 'STRG'-Taste).
     * 
     * @param e das MouseEvent, aus das reagiert wird.
     * @see java.awt.event.MouseMotionListener
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent e) {
        // 3D-Kamera...
        
            if (rightMousePressed) {
                int transformateX = e.getX() - mouseX;
                int transformateY = mouseY - e.getY();
                if (strgPressed) {
                     cam.rotateLocal((float) -transformateY,
                            (float) transformateX, 0);
                } else {
                    // 3D-Kamera verschieben
                    cam.translateLocal((float) transformateX / 100,
                            (float) transformateY / 100, 0);
                }
            }
            mouseX = e.getX();
            mouseY = e.getY();
    }

    /**
     * Reagiert auf das Mausrad mit Kamerabewegungen auf der
     * lokalen z-Achse.
     * 
     * @param e das MousewheelEvent, auf das reagiert wird.
     * @see java.awt.event.MouseWheelListener
     * @see java.awt.event.MouseWheelListener#mouseWheelMoved(java.awt.event.MouseWheelEvent)
     */
    public void mouseWheelMoved(MouseWheelEvent e) {
       
        // 3D-Kamera verschieben
        cam.translateLocal(0, 0, e.getWheelRotation() * 1.0f);
        
        mouseX = e.getX();
        mouseY = e.getY();
    }

    /**
     * Leer.
     * 
     * @param e das MouseEvent
     * @see java.awt.event.MouseMotionListener
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    public void mouseMoved(MouseEvent e) {

    }

    /**
     * Reagiert auf Maustasten-Eigaben.
     * 
     * <p>Diese Eigaben sind:<br>
     * &bull; rechte Maustaste -> Kamerabewegung</p>
     * 
     * @param e das MouseEvent, auf das reagiert wird
     * @see java.awt.event.MouseEvent
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e) {
        // Rechte Maustaste -> Kamerabewegung
        if (e.getButton() == 3) {
            this.rightMousePressed = true;
        }

        this.mouseX = e.getX();
        this.mouseY = e.getY();
    }

    /**
     * Reagiert auf das Loslassen einer Maustaste.
     * 
     * <p>Das Loslassen der Maustaste gibt den Endpunkt des
     * Selektionsfeldes an, welches dann gezeichnet wird.</p>
     * 
     * @param e das MouseEvent, auf das reagiert werden soll
     * @see java.awt.event.MouseEvent
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     * 
     */
    public void mouseReleased(MouseEvent e) {
        if (e.getButton() == 3)
            this.rightMousePressed = false;
    }

    /**
     * Liefert die aktuelle x-Position des Mausklicks zur&uuml;ck.
     * 
     * @return die x-Position des Mausklicks
     * @see java.lang.Float
     */
    public float getMouseX() {
        return mouseClickX;
    }

    /**
     * Liefert die aktuelle y-Position des Mausklicks zur&uuml;ck.
     * 
     * @return die y-Position des Mausklicks
     * @see java.lang.Float
     */
    public float getMouseY() {
        return mouseClickY;
    }

   
    /**
     * Reagiert auf Benutzereingaben in Form von Mausklicks.
     * 
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent e) {
        
    }

    /**
     * Setzt das Canvas in den Fokus, damit KeyEvents und
     * MouseEvents empfangen werden k&ouml;nnen.
     * 
     * @see java.awt.event.MouseListener#mouseEntered
     * @see java.awt.event.MouseEvent
     */
    public void mouseEntered(MouseEvent e) {
        // Canvas in Fokus setzen, damit KeyEvents empfangen werden knnen
        canvas.requestFocusInWindow();
    }

    /**
     * 
     * 
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent arg0) {
    }

    /**
     * Reagiert auf Benutzereingaben in Form von Tastendruck.
     * 
     * <p>Auf folgende Tasten wird reagiert:
     * <ul>
     * <li>'strg'   -> Feinschrittigere Bewegung</li>
     * <li>'alt'    -> Setzt das entsprechende Flag auf <code>true</code></li>
     * <li>'shift'  -> Setzt das entsprechende Flag auf <code>true</code></li>
     * <li>'L' -> Schaltet das Licht ein bzw macht es heller</li>
     * </ul></p>
     * 
     * @param e das <code>KeyEvent</code>, auf das reagiert werden soll
     * @see java.awt.event.KeyListener
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    public void keyPressed(KeyEvent e) {

        if (e.getKeyCode() == KeyEvent.VK_CONTROL) {
            strgPressed = true;
        }

        if (e.getKeyCode() == KeyEvent.VK_DOWN) {}
        if (e.getKeyCode() == KeyEvent.VK_UP) {}
        if (e.getKeyCode() == KeyEvent.VK_LEFT) {}
        if (e.getKeyCode() == KeyEvent.VK_RIGHT) {}
        if (e.getKeyCode() == KeyEvent.VK_L){
            GLSettings.updateAmbientLight(0.2f);
        }

    }

    /**
     * <b>keyReleased</b><br>
     * <br>
     * <code>public void <b>keyReleased</b>(KeyEvent e)</code><br>
     * <br>
     * Diese Methode reagiert auf Loslassen der <i>strg</i>-Taste
     * 
     * @param e -
     *            das <code>KeyEvent</code>, auf das reagiert werden soll
     * @see java.awt.event.KeyListener
     * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
     */
    public void keyReleased(KeyEvent e) {
        
        if (e.getKeyCode() == KeyEvent.VK_CONTROL) {
            strgPressed = false;
        }
    }

    public void keyTyped(KeyEvent e) {
       
    }

}
    
    

