package big3d.material;

import java.util.ArrayList;

import big3d.utils.Color;


/**
 * Diese Klasse hier beinhaltet alles, was man fr Material braucht:
 * Materialeigenschaften, Texturen
 * @author derIch
 *
 */
public class Material{
	//transparent?
    private boolean transparent;
    
    //Farbe:
    private Color color;
    
    // Textur(en)
    public ArrayList<Integer> textures = new ArrayList<Integer>();
    
    //Normal map (fr bumpmapping)
    public int normalMap;

    // abstrahlung des ambienten Lichts (nach Konvention = diffuse)
    public float[] ambient = { 0.2f, 0.2f, 0.2f, 1.0f };
    
    // reflexion des Lichtstrahls
    protected float[] specular = { 0.0f, 0.0f, 0.0f, 1.0f };

    // selbstabgabe von licht:
    protected float[] emission = { 0.0f, 0.0f, 0.0f, 1.0f };
    
    // Exponent der Shininess (Glanzheit) (0-100)
    protected float shininess = 20.0f;
    
    /**
     * leerer Konstruktor
     *
     */
    public Material(){
        this.transparent = false;
        this.color = new Color();
    }

    
    /**
     * Copy-Konstruktor 
     */
    public Material(Material mo){
        for (int i = 0; i < mo.ambient.length; i++) {
            this.ambient[i] = mo.ambient[i];
        }
        
        for (int i = 0; i < mo.specular.length; i++) {
            this.specular[i] = mo.specular[i];
        }
        
        for (int i = 0; i < mo.emission.length; i++) {
            this.emission[i] = mo.emission[i];
        }
        
        this.normalMap = mo.getNormalMap();
         
        this.shininess = mo.shininess;
                
        this.transparent = mo.isTransparent();
    }
   
    /**
     * @return Returns the ambient.
     */
    public float[] getAmbient() {
        return ambient;
    }



    /**
     * @return Returns the emission.
     */
    public float[] getEmission() {
        return emission;
    }


    /**
     * @return Returns the shininess.
     */
    public float getShininess() {
        return shininess;
    }

    /**
     * @return Returns the specular.
     */
    public float[] getSpecular() {
        return specular;
    }

    /**
     * @return Returns the textures.
     */
    public ArrayList<Integer> getTextures() {
        return textures;
    }

    /**
     * @param ambient The ambient to set.
     */
    public void setAmbient(float[] ambient) {
        this.ambient = ambient;
    }


    /**
     * @param color The ambientcolor to set.
     */
    public void setAmbientAndDiffuse(Color color) {
        this.ambient = color.getColor();
    }
    
    public void setColor(Color color){
        if (color.getColor()[3] != 1.0f){
            float newVal = color.getColor()[3];
            this.ambient[3] = newVal;
            this.color = color;
            this.transparent = true;
        } 
    }
        
        
    public void setTransperancy(float percent){
        if (percent != 0){
            this.color.getColor()[3] = 1.0f - (percent/100);
            this.ambient[3] = 1.0f - (percent/100);
            this.transparent = true;
        } else this.transparent = false;
    }
    
    public void setSingleColor(Color color) {
        if (color.getColor()[3] != 1.0f){
            float newVal = color.getColor()[3];
            this.ambient[3] = newVal;
            this.transparent = true;
        }
    }



    /**
     * @param emission The emission to set.
     */
    public void setEmission(float[] emission) {
        this.emission = emission;
    }


    /**
     * @param shininess The shininess to set.
     */
    public void setShininess(float shininess) {
        this.shininess = shininess;
    }


    /**
     * @param specular The specular to set.
     */
    public void setSpecular(float[] specular) {
        this.specular = specular;
    }
    
  
    
    /**
     * Fgt eine Textur (bzw. deren ID) dem Material hinzu.
     * @param texID
     */
    public void addTexture(int texID){
        // added die Textur:
        this.textures.add(texID);
    }
    
    public void setTexture(int texID, int index){
        //berschreibt die gewnschte Textur
        this.textures.add(index,texID);
    }

   
    /**
     * @return Returns the normalMap.
     */
    public int getNormalMap() {
        return normalMap;
    }

    /**
     * @param normalMap The normalMap to set.
     */
    public void setNormalMap(int normalMap) {
        this.normalMap = normalMap;
    }
    
    public void deleteNormalMap(){
       this.normalMap = -1;
    }
    
        public void deleteTexture(int textureIndex){
            this.textures.remove(textureIndex);
            if(this.textures.size() == 0)
            {}//TODO
            
            if(this.textures.size() == 1)
            {}  
        }
        
        /**
         * @return Returns the transparent.
         */
        public boolean isTransparent() {
            return transparent;
        }


        /**
         * @return Returns the color.
         */
        public Color getColor() {
            return color;
        }        
        
}
