/*
 * Created on 22.02.2006
 *
 */
package big3d.material;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import javax.imageio.ImageIO;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;


/**
 * Hier sollen Texturen von der Festplatte geladen werden. 
 * Alle untersttzten Formate mssen hier implementiert sein.
 * 
 * @author Lars Lttmann
 */
public class TextureLoader {
    //"das Bild"
    private static ByteBuffer pixel;
    
	  /**
       *  Retrieve a URL resource from the jar.  If the resource is not found, then
	   * the local disk is also checked.
	   * @param filename Complete filename, including parent path
	   * @return a URL object if resource is found, otherwise null.
	   */  
	public final static URL getResource(final String filename)
	  {
	    // Try to load resource from jar
	    URL url = ClassLoader.getSystemResource(filename);
	    // If not found in jar, then load from disk
	    if (url == null)
	    {
	      try
	      {
	        url = new URL("file", "localhost", filename);
	      }
	      catch (Exception urlException){} // ignore
	    }
	    return url;
	}
	
    
	/**
     *  Ldt und bestimmt alle Parameter der Textur 
     *  (holt sich diese aus dem TextureObjekt).
     * @param gl
     * @param glu
     * @param resourceName
     * @param texO
     * @param mipmap
	 */
    public static void LoadTexture(GL gl, GLU glu, String resourceName, TextureObject texO,
            boolean mipmap) {
        BufferedImage image;
        
        //lschen des "alten" Bildes
        pixel =null;
        
        // Einlesen der Datei und spiegeln des Bildes
        try {
            String originalResourceName = resourceName;
            
            if (!(new File(resourceName).canRead())) {
                resourceName = originalResourceName.substring(6);
            }
            
            if (!(new File(resourceName).canRead())) {
                resourceName = "./src/".concat(originalResourceName);
            }
            
            image = ImageIO.read(new File(resourceName));
                if (image ==null){
                 //   System.out.println("MOPES");
                    System.out.println("HALLO!!: "+resourceName);
                }
            
            AffineTransform translatex = AffineTransform
                    .getScaleInstance(1, -1);
            translatex.translate(0, -image.getHeight());
            AffineTransformOp op = new AffineTransformOp(translatex,
                    AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
            
            image = op.filter(image, null);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }   
        
        // bestimmen von Hhe und Breite
        int height = image.getHeight();
        int width = image.getWidth();
        
        
        BufferedImage buffered = new BufferedImage( width,
				height,BufferedImage.TYPE_3BYTE_BGR);
		Graphics2D g2 = buffered.createGraphics();
		g2.drawImage( image, null, null );
		image = buffered;
        
        byte[] bytedata = ((DataBufferByte) image.getRaster()
                .getDataBuffer()).getData();
        pixel = ByteBuffer.allocateDirect(bytedata.length);
        pixel.order(ByteOrder.nativeOrder());
        pixel.put(bytedata, 0, bytedata.length);
               
        putTexture2OpenGL(mipmap, glu, gl, width, height, pixel);
    }

    public static void putTexture2OpenGL(boolean mipmap, GLU glu, GL gl, int width, int height, ByteBuffer pixel){
        if (mipmap)
        {
            glu.gluBuild2DMipmaps(GL.GL_TEXTURE_2D, GL.GL_RGB, width, height,  GL.GL_BGR, GL.GL_UNSIGNED_BYTE, pixel);
        }
        else
        {
            gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, width, height, 1, GL.GL_BGR, GL.GL_UNSIGNED_BYTE, pixel);
            System.out.println("Breite : "+width);
            if (gl.glGetError() != 0) {
                System.out.println("Breite : "+width);
                gl.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, width, height, 0, GL.GL_BGR, GL.GL_UNSIGNED_BYTE, pixel);
            }
        }
    }
    
	// Methode aus dem TextureLoader von XITH3D
	public static int getClosestPowerOf2(int value) {

        if (value < 1) {
            return value;
        }

        int powerValue = 1;
        for (; ; ) {
            powerValue *= 2;
            if (value == powerValue)
                return powerValue;
            if (value < powerValue) {
                // Found max bound of power, determine which is closest
                int minBound = powerValue / 2;
                if ( (powerValue - value) >
                    (value - minBound)) {
                    return minBound;
                } else {
                    return powerValue;
                }
            }
        }
    }
}
