package big3d.material;

import java.nio.ByteBuffer;

import net.java.games.jogl.*;

/**
 * <b>TextureObject</b><br><br>
 * <code>public class <b>TextureObject</b></code><br><br>
 * Diese Klasse realisiert ein <code>NewTextureObjekt</code>.
 * Das sind Texturen,
 * mit denen Geometrie-Objekte versehen werden k&ouml;nnen.<br>
 * Diese Klasse ersetzt die alte Klasse <code>TextureObject</code>.
 * Grundlage f&uuml;r eine Textur ist mindestens eine Grafik-Datei.<br><br> 
 * <b>private Variablen</b>:<br>
 * <code>private int <b>ID</b></code> - Die ID der Textur<br>
 * <code>private String <b>path</b></code> - Pfad der Texturdatei
 * <code>private String <b>filename</b></code> - Dateiname der Texturdatei
 * @author Lars L&uuml;ttmann
 */
public class TextureObject{
    //id der Textur (intern)
    private int ID=-1;
    
    //ID der sobald sie von OpenGL geladen wurde. 
    private int myOpenGLID = -1;

    //speichert den Namen einer Datei (inkl. Endung):
    private String filename = null;
    
    //ALTERNATIV:
    private ByteBuffer texture;
    private int dim;
    
    /* ----------- Initialisierung ----------- */
    
    /**
     * <b>TextureObject</b> Konstruktor<br><br>
	 * <code>public <b>TextureObject</b>(String path)</code><br><br>
	 * Initialisierung des <code>NewTextureObjects</code> mit einer Grafik-Datei
	 * als Basis.
     * @param filename - Filename der Grafik-Datei
     */
    public TextureObject(String filename){
        this.filename = filename;
    }
    
    public TextureObject(){
    	
    }
    
    public TextureObject(ByteBuffer tex){
        this.texture = tex;
        this.dim = getDim(tex);
    }
   
    public TextureObject(TextureObject object) {
        this.ID = object.getID();
               
        this.myOpenGLID = -1;

        this.filename = object.getFilename();
    }

    public void initTexture(GLDrawable gld) {
    	
    	if (this.myOpenGLID==-1){
            GL gl = gld.getGL();
    		GLU glu = gld.getGLU();
    		
    		gl.glGetError();
    		
    		// aktivieren der Textur:
    		int[] temp = new int[1];
    		gl.glGenTextures(1, temp);
    		
    		//Textur binden:     
    		gl.glBindTexture(GL.GL_TEXTURE_2D, temp[0]);
    		
    		//merken der ID:
    		this.setMyOpenGLID(temp[0]);
    		
            //ldt entweder eine Textur aus einem File oder... 
            if(this.filename!=null && this.texture == null){
                TextureLoader.LoadTexture(gl,glu,filename,this,true);
            } else{
                //eine selbstdefinierte
                TextureLoader.putTexture2OpenGL(true, glu, gl, dim, dim, texture);
            }
            
    		//fertig: vorm Zeichnen sollte dann gl.glBindTexture(myOpenGLID); gesetzt werden.
    	}
    }
    
    
    /* ----------- set- und get-methoden ----------- */
    
    /**
     * <b>getID</b><br><br>
	 * <code>public int <b>getID</b>()</code><br><br>
	 * Diese Methode liefert die aktuelle ID des <code>NewTextureObjects</code>.
	 * Diese ID ist...
     * @return Returns the iD.
     */
    public int getID() {
        return ID;
    }

    
  
    /**
     * <b>setID</b><br><br>
	 * <code>public void <b>setID</b>(int id)</code><br><br>
	 * Diese Methode legt eine neue ID des <code>NewTextureObjects</code> fest.
	 * Diese ID ist...
     * @param id - die neue ID
     */
    public void setID(int id) {
        ID = id;
    }


    /**
     * @return Returns the filename.
     */
    public String getFilename() {
        return filename;
    }

    /**
     * @param filename The filename to set.
     */
    public void setFilename(String filename) {
        this.filename = filename;
    }

    

    /**
     * @return Returns the myOpenGLID.
     */
    public int getMyOpenGLID() {
        return myOpenGLID;
    }


    /**
     * @param myOpenGLID The myOpenGLID to set.
     */
    public void setMyOpenGLID(int myOpenGLID) {
        this.myOpenGLID = myOpenGLID;
    }

    private int getDim(ByteBuffer bb){
        if (bb.capacity() == 2)
            return 1;
        else {
            int dim_temp = (int) Math.sqrt((double) bb.capacity());
            return TextureLoader.getClosestPowerOf2(dim_temp);
        }
    }
}
