/*
 * Created on 08.01.2006
 *
 */
package big3d.objects;

import net.java.games.jogl.*;
/**
 * <b>FogObject</b><br><br>
 * <code>public class <b>FogObject</b></code><br><br>
 * Diese Klasse ist eine Datenklasse f&uuml;r Nebel-Objekte. Die eigentliche
 * Erzeugung der Nebel-Objekte geschieht in der <code>FogFactory</code>.<br><br>
 * <b>Globale Variablen</b>:<br>
 * <code>private int[] fogModes</b></code> - die drei verschiedenen Nebel-Modi
 * aus <code>GL_EXP<code> (f&uuml;r &auml;ltere PCs), <code>GL_EXP2</code>
 * (schon besser) und <code>GL_LINEAR</code> (am Besten)<br>
 * <code>private int fogMode</b></code> - der f&uuml;r dieses Objekt 
 * gew&auml;hlte Modus<br>
 * <code>private float[] fogColor</b></code> - Farbe des Nebels<br>
 * <code>private float fogDensity</b></code> - Dichte des Nebels<br>
 * <code>private float fogStart</b></code>Wo startet der Nebel? (vom Betrachter)<br>
 * <code>private float fogEnd</b></code>Wo endet der Nebel? (vom Betrachter)<br>
 * @author Lars L&uuml;ttmann
 */
public class FogObject {
    //die drei verschiedenen Nebel-Modi (EXP fr alte PCs, EXP2 schon besser, LINEAR am Besten
    private int[] fogModes = {GL.GL_EXP, GL.GL_EXP2, GL.GL_LINEAR};
    
    //der fr dieses Object gewhlte modi (def. EXP2): 
    private int fogMode = fogModes[2];
    
    //Farbe des Nebels
    private float[] fogColor= {0.2f, 0.2f, 0.2f, 1f};
    
    //Dichte des Nebels:
    private float fogDensity = 0.35f;
    
    //Wo startet der Nebel? (vom betrachter)
    private float fogStart = 300f;
    
    //Wo endet der Nebel? (vom betrachter)
    private float fogEnd = 420f;
    
    /**
     * <b>FogObject</b> - Konstruktor<br><br>
	 * <code>public <b>FogObject</b>()</code><br><br>
     * Leerer <code>default</code>-Konstruktor zur Initialisierung des
     * Nebel-Objekts.
     */
    public FogObject(){   
    }
    /**
     * <b>draw</b><br><br>
	 * <code>public void <b>draw</b>(GL gl)</code><br><br>
	 * Mit dieser Methode kann sich das Nebel-Objekt selbst zeichnen.
     * @param gl - das von OpenGL zum Zeichnen erforderte <code>GL</code>-Objekt
     */
    public void draw(GL gl){
        gl.glFogi(GL.GL_FOG_MODE, fogMode); // Fog Mode
        gl.glFogfv(GL.GL_FOG_COLOR, fogColor); // Set Fog Color
        gl.glFogf(GL.GL_FOG_DENSITY, fogDensity); // How Dense Will The Fog Be
        gl.glHint(GL.GL_FOG_HINT, GL.GL_DONT_CARE); // Fog Hint Value
        gl.glFogf(GL.GL_FOG_START, fogStart); // Fog Start Depth
        gl.glFogf(GL.GL_FOG_END, fogEnd); // Fog End Depth
        gl.glEnable(GL.GL_FOG); // Enables GL_FOG
    }

    /* ----------- getter und setter ---------- */
    
    /**
     * <b>getFogColor</b><br><br>
	 * <code>public float[] <b>getFogColor</b>()</code><br><br>
	 * Diese Methode liefert die aktuelle Farbe des Nebel-Objekts.
     * @return die Farbe
     */
    public float[] getFogColor() {
        return fogColor;
    }

    /**
     * <b>setFogColor</b><br><br>
	 * <code>public void <b>setFogColor</b>(float[] fogColor)</code><br><br>
	 * Diese Methode legt eine neue Farbe f&uuml;r das Nebel-Objekt fest.  
     * @param fogColor  - die neue Farbe
     */
    public void setFogColor(float[] fogColor) {
        this.fogColor = fogColor;
    }

    /**
     * <b>getFogDensity</b><br><br>
	 * <code>public float <b>getFogDensity</b>()</code><br><br>
	 * Diese Methode liefert die aktuelle Dichte des Nebel-Objekts.
     * @return die Dichte
     */
    public float getFogDensity() {
        return fogDensity;
    }

    /**
     * <b>setFogDensity</b><br><br>
	 * <code>public void <b>setFogDensity</b>(float fogDensity)</code><br><br>
	 * Diese Methode legt eine neue Dichte f&uuml;r das Nebel-Objekt fest.
     * @param fogDensity  - die neue Dichte
     */
    public void setFogDensity(float fogDensity) {
        this.fogDensity = fogDensity;
    }

    /**
     * <b>getFogEnd</b><br><br>
	 * <code>public float <b>getFogEnd</b>()</code><br><br>
	 * Diese Methode liefert das aktuelle Ende des Nebel-Objekts (vom
	 * Betrachter aus gesehen).
     * @return das Ende
     */
    public float getFogEnd() {
        return fogEnd;
    }

    /**
     * <b>setFogEnd</b><br><br>
	 * <code>public void <b>setFogEnd</b>(float fogEnd)</code><br><br>
	 * Diese Methode legt ein neues Ende (vom Betrachter aus gesehen) f&uuml;r
	 * das Nebel-Objekt fest.
     * @param fogEnd - das neue Ende
     */
    public void setFogEnd(float fogEnd) {
        this.fogEnd = fogEnd;
    }

    /**
     * <b>getFogMode</b><br><br>
	 * <code>public int <b>getFogMode</b>()</code><br><br>
	 * Diese Methode liefert den aktuellen Modus.
     * @return den Modus aus <code>GL_EXP</code>,
	 * <code>GL_EXP2</code> und <code>GL_LINEAR</code>
     */
    public int getFogMode() {
        return fogMode;
    }

    /**
     * <b>setFogMode</b><br><br>
	 * <code>public void <b>setFogMode</b>(int fogMode)</code><br><br>
	 * Diese Methode legt den neuen Modus fest.
     * @param fogMode - Modus: <code>GL_EXP</code>,
	 * <code>GL_EXP2</code> oder <code>GL_LINEAR</code>
     */
    public void setFogMode(int fogMode) {
        this.fogMode = fogMode;
    }

    /**
     * <b>getFogModes</b><br><br>
	 * <code>public int[] <b>getFogModes</b>()</code><br><br>
	 * Diese Methode liefert die aktuelle Liste mit den Nebel-Modi.
     * @return die Liste mit den Modi
     */
    public int[] getFogModes() {
        return fogModes;
    }

    /**
     * <b>setFogModes</b><br><br>
	 * <code>public void <b>setFogModes</b>(int[] fogModes)</code><br><br>
	 * Diese Methode legt die Nebel-Modi fest.
     * @param fogModes - das Array mit den Nebel-Modi
     */
    public void setFogModes(int[] fogModes) {
        this.fogModes = fogModes;
    }

    /**
     * <b>getFogStart</b><br><br>
	 * <code>public float <b>getFogStart</b>()</code><br><br>
	 * Diese Methode liefert den Startpunkt des Nebel-Objekts (vom Betrachter aus
	 * gesehen).
     * @return den Anfang
     */
    public float getFogStart() {
        return fogStart;
    }

    /**
     * <b>setFogStart</b><br><br>
	 * <code>public void <b>setFogStart</b>(float fogStart)</code><br><br>
	 * Diese Methode legt den neuen Anfang des Nebel-Objekts (vom Betrachter
	 * aus gesehen) fest.
     * @param fogStart - der neue Anfang
     */
    public void setFogStart(float fogStart) {
        this.fogStart = fogStart;
    }
}
