/*
 * Created on 01.05.2006
 *
 */
package big3d.objects;

import java.util.ArrayList;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import big3d.material.Material;
import big3d.material.MultiTextureHelper;
import big3d.material.TextureLib;
import big3d.surface.Surface;
import big3d.utils.Deforming;
import big3d.utils.GLSettings;
import big3d.utils.GLVector;
import big3d.utils.MathHelper;
import big3d.utils.TexCoord;
import big3d.utils.Vertex;

public class Mesh {
    //ein paar Listen:
    public Material material;
    public ArrayList<GLVector> normals;
    public ArrayList<Vertex> edges;
    public ArrayList<Surface> surfaces;
    public ArrayList<Mesh> meshes;
    public ArrayList<TexCoord> texCoords;
    
    public boolean roundNormals = false;
    
//  zeigt an, ob die Texturcoordinaten von OGL gemacht werden sollen
    public boolean autoTexCoords = true; 
    
    private float[] position = new float[3]; //absolute position des Objektes
    private float[] rotate = {0.0f , 0.0f, 0.0f}; //rotation um jede Achse
    
    public float shearY = 0f;
    
    public Mesh(Mesh other){
        this();
        for (int i=0; i < other.surfaces.size(); i++){
            this.surfaces.add(other.surfaces.get(i));
            this.setPosition(other.position);
            this.autoTexCoords = other.autoTexCoords;
        }
        
    }
    public Mesh(){
        edges = new ArrayList<Vertex>();
        normals = new ArrayList<GLVector>();
        surfaces = new ArrayList<Surface>();
        texCoords = new ArrayList<TexCoord>();
        this.material = new Material();
    }
    
    public void draw(GLDrawable gld){
        GL gl = gld.getGL();
        
        //Fehlerzustand resetten:
        gl.glGetError();
        
        gl.glPushMatrix();
        
        gl.glTranslatef(position[0], position[1], position[2]);
        
        //scheren? TODO raus?
        if (shearY!=0)
            Deforming.shearY(gl, shearY);
        
        // rotation:
        if (rotate[0] != 0 || rotate[1] != 0 || rotate[2] != 0)
            MathHelper.rotate(gl, rotate[0], rotate[1], rotate[2]);
        
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Rotieren des Meshes!");
        
        
        /**----------------- ZEICHNEN: -------------**/
        //Material setzen:
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE, this.material.getAmbient());
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, this.material.getSpecular());
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_EMISSION, this.material.getEmission());
        gl.glMaterialf(GL.GL_FRONT, GL.GL_SHININESS, this.material.getShininess());
        
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Setzen des Materials des Meshes!");
        
        
        //maximale Texturen, die diese Karte kann:
        int maxTex = GLSettings.maxTex;
        
        
        //Je nach Texturenlage wird die entsprechende Draw-Methode genommen:
        if(this.material.textures.size() != 0){
            if (this.autoTexCoords){
                this.drawWithAutoTextureCoords(maxTex, gld);
            } else {
                this.drawWithoutAutoTextureCoords(maxTex, gld);
            }
            
        } else this.drawWithoutTextures(gl); 
 
        gl.glPopMatrix();
    }
    
    private void drawWithAutoTextureCoords(int maxTex, GLDrawable gld){
        //System.err.println("Zeichne mit automatische Texturkoordinaten!");
        
        GL gl = gld.getGL();
        
        //Idee: nicht unntig arbeiten:
        if (maxTex > this.material.textures.size())
            maxTex=this.material.textures.size();
        
        //alle Texturen bereitstellen: 
        for (int i= 0; i<maxTex;i++){
            //aktivieren eines Texturkanals:
            gl.glActiveTexture(MultiTextureHelper.getTexture(i));
            
            //TexturID holen 
            int texID = TextureLib.getInstance().getTexture(this.material.textures.get(i).
                    intValue()).getMyOpenGLID(); 
            
            //Texture in den entsprechenden Texturkanal schieben
            gl.glBindTexture(GL.GL_TEXTURE_2D, texID);
            
            gl.glEnable(GL.GL_TEXTURE_2D);
            if (gl.glGetError() != GL.GL_NO_ERROR)
                System.err.println("Fehler beim Setzen einer Textur!");
            
            //Texturumgebung setzen!
            GLSettings.setDefaultTextureEnv(gl); 
            if (gl.glGetError() != GL.GL_NO_ERROR)
                System.err.println("Fehler beim Setzen der TextureEnviroment!");
            
            // automatische Texturkoordinatenberechnung:
            gl.glTexGeni(GL.GL_S, GL.GL_TEXTURE_GEN_MODE, GL.GL_OBJECT_LINEAR);
            gl.glTexGeni(GL.GL_T, GL.GL_TEXTURE_GEN_MODE, GL.GL_OBJECT_LINEAR);
            
            //und einschalten:
            gl.glEnable(GL.GL_TEXTURE_GEN_S);
            gl.glEnable(GL.GL_TEXTURE_GEN_T);
        }
        
        // setzen der Farbe des Objektes:
        gl.glColor4fv(this.material.getColor().getColor());
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Setzen der Farbe des Meshes!");
        
        // ZEICHNEN der einzelnen surfaces:
        for (int i=0; i< surfaces.size(); i++){
            surfaces.get(i).drawNoTex(gl);
        }
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Zeichnen der Surfaces des Meshes!");
        
        //Bei Texturieren die entsprechenden (wenn keine: maxTex ist 0)
        for (int i= 0; i<maxTex;i++){
            //aktivieren eines Texturkanals:
            gl.glActiveTexture(MultiTextureHelper.getTexture(i));
            
            //Texturieren wieder ausschalten:
            gl.glDisable(GL.GL_TEXTURE_2D);
            
            // automatische Texturkoordinatenberechnung ausschalten:
            gl.glDisable(GL.GL_TEXTURE_GEN_S);
            gl.glDisable(GL.GL_TEXTURE_GEN_T);
        }
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Zurcksetzen der Texturen des Meshes!");

    }
    
    private void drawWithoutAutoTextureCoords(int maxTex, GLDrawable gld){
       // System.err.println("Zeichne ohne automatische Texturkoordinaten!");
        GL gl = gld.getGL();
        
        //Idee: nicht unntig arbeiten:
        if (maxTex > this.material.textures.size())
            maxTex=this.material.textures.size();
        
        //alle Texturen bereitstellen: 
        for (int i= 0; i<maxTex;i++){
            //aktivieren eines Texturkanals:
            gl.glActiveTexture(MultiTextureHelper.getTexture(i));
            
            //TexturID holen 
            int texID = TextureLib.getInstance().getTexture(this.material.textures.get(i).
                    intValue()).getMyOpenGLID(); 
            
            //Texture in den entsprechenden Texturkanal schieben
            gl.glBindTexture(GL.GL_TEXTURE_2D, texID);
            
            gl.glEnable(GL.GL_TEXTURE_2D);
            if (gl.glGetError() != GL.GL_NO_ERROR)
                System.err.println("Fehler beim Setzen einer Textur!");
            
            //Texturumgebung setzen!
            GLSettings.setDefaultTextureEnv(gl); 
            if (gl.glGetError() != GL.GL_NO_ERROR)
                System.err.println("Fehler beim Setzen der TextureEnviroment!");
          
        }
        
//      setzen der Farbe des Objektes:
        gl.glColor4fv(this.material.getColor().getColor());
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Setzen der Farbe des Meshes!");
        
        // ZEICHNEN der einzelnen surfaces:
        for (int i=0; i< surfaces.size(); i++){
            surfaces.get(i).draw(gld, maxTex);
        }
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Zeichnen der Surfaces des Meshes!");
        
        //Bei Texturieren die entsprechenden (wenn keine: maxTex ist 0)
        for (int i= 0; i<maxTex;i++){
            //aktivieren eines Texturkanals:
            gl.glActiveTexture(MultiTextureHelper.getTexture(i));
            
            //Texturieren wieder ausschalten:
            gl.glDisable(GL.GL_TEXTURE_2D);
        }
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Zurcksetzen der Texturen des Meshes!");

    }
    
    private void drawWithoutTextures(GL gl){
       // System.err.println("Zeichne ohne Texturen!");
        
        // setzen der Farbe des Objektes:
        gl.glColor4fv(this.material.getColor().getColor());
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Setzen der Farbe des Meshes!");
        
        // ZEICHNEN der einzelnen surfaces:
        for (int i=0; i< surfaces.size(); i++){
            surfaces.get(i).drawNoTex(gl);
        }
        if (gl.glGetError() != GL.GL_NO_ERROR)
            System.err.println("Fehler beim Zeichnen der Surfaces des Meshes!");
        
    }
 
    public void addSurface(Surface surface){
        surfaces.add(surface);
    }

    /**
     * @return Returns the position.
     */
    public float[] getPosition() {
        return position;
    }

    /**
     * @param position The position to set.
     */
    public void setPosition(float[] position) {
        this.position = position;
    }
    
    /**
     * @param x
     * @param y
     * @param z
     */
    public void setPosition(float x, float y, float z) {
        this.position[0] = x;
        this.position[1] = y;
        this.position[2] = z;
    }

    /**
     * @return Returns the rotate.
     */
    public float[] getRotate() {
        return rotate;
    }
    
    
    /**
     * @param rotate The rotate to set.
     */
    public void setRotate(float[] rotate) {
        this.rotate = rotate;
    }
    
    /**
     * @param x
     * @param y
     * @param z
     */
    public void setRotate(float x, float y, float z) {
        this.rotate[0] = x;
        this.rotate[1] = y;
        this.rotate[2] = z;
    }
    
    public int addEdge(Vertex edge){
        int index = this.edges.size();
        this.edges.add(edge);
        //TODO: doppelte Ecken ignorieren (Rckgabe beachten!)
        return index;
    }
    
    public GLVector getNormal(int index){
        return this.normals.get(index);
    }
    
    public Vertex getVertex(int index){
        return this.edges.get(index);
    }
    
    /**
     * Wird zum setzen von runden Normalen gebraucht.
     * @param normal
     * @param pos
     */
    public void addNormal(GLVector normal, int[] pos){
        if(pos.length!=1){
            for(int i=0 ; i < pos.length; i++){
                try {
                    if (normals.get(pos[i])== null){
                        // addieren und normieren mit dem vorhandenen Vektor
                        GLVector n1 = MathHelper.vectorAddVector(this.normals.get(pos[i]), normal);
                        this.normals.add(pos[i], MathHelper.normalizeVector(n1));
                    }
                }catch (IndexOutOfBoundsException e){
                    //offenbar noch leer:
                    this.normals.add(pos[i], normal);
                }
            }
        }else{
            this.normals.add(pos[0], normal);
        }
    }
    
    public void deleteNormals(){
        this.normals.clear();
        for(int m=0; m<edges.size(); m++){
            this.normals.add(null);
        }
    }
    
    /**
     * @return Returns the roundNormals.
     */
    public boolean isRoundNormals() {
        return roundNormals;
    }
    /**
     * @param roundNormals The roundNormals to set.
     */
    public void setRoundNormals(boolean roundNormals) {
        this.roundNormals = roundNormals;
    }
    
    public void addMesh(Mesh mesh){
        this.meshes.add(mesh);
    }
    
    public TexCoord getTexCoord(int id){
        return this.texCoords.get(id);
    }
    
    /**
     * @return Returns the material.
     */
    public Material getMaterial() {
        return material;
    }
    
    /**
     * @param material The material to set.
     */
    public void setMaterial(Material material) {
        this.material = material;
    }
    
    public void update(){
        //TODO
    }
    
    public int addTextureCoord(TexCoord coord){
        int result = this.texCoords.size();
        this.texCoords.add(coord);
        
        return result;
    }
}
