/*
 * Created on 02.05.2006
 *
 */
package big3d.objects;

import big3d.surface.Quad;
import big3d.surface.Triangle;
import big3d.utils.GLVector;
import big3d.utils.MathHelper;
import big3d.utils.TexCoord;
import big3d.utils.Vertex;

public class MeshFactory {
    
    // Quadratische Texturkoordinaten definieren:
    public static TexCoord lu = new TexCoord(0f, 0f); //links unten
    public static TexCoord ru = new TexCoord(1f, 0f); //rechts unten
    public static TexCoord ro = new TexCoord(1f, 1f); //rechts oben
    public static TexCoord lo = new TexCoord(0f, 1f); //links oben
    public static TexCoord om = new TexCoord(0.5f, 1f); // oben mitte
    
    /**
     * 
     * @param height
     * @param length
     * @param depth
     * @return
     */
    public static Mesh createCuboid(float height, float length, float depth){
        Vertex v1 = new Vertex(-length/2, -height/2, depth/2); //vorne unten-l
        Vertex v2 = new Vertex(length/2, -height/2, depth/2); //vorne u-r
        Vertex v3 = new Vertex(length/2, height/2, depth/2); //vorne oben-r
        Vertex v4 = new Vertex(-length/2, height/2, depth/2); //vorne oben-l
        Vertex v5 = new Vertex(length/2, -height/2, -depth/2); //hinten unten-r
        Vertex v6 = new Vertex(-length/2, -height/2, -depth/2); //hinten unten-l
        Vertex v7 = new Vertex(-length/2, height/2, -depth/2); //hinen oben-l
        Vertex v8 = new Vertex(length/2, height/2, -depth/2); //hinten oben-r
        
        //erstelle das Mesh:
        Mesh cuboid = new Mesh();
        
        //eckpunkte hinzufgen
        int a1 = cuboid.addEdge(v1);
        int a2 = cuboid.addEdge(v2);
        int a3 = cuboid.addEdge(v3);
        int a4 = cuboid.addEdge(v4);
        int a5 = cuboid.addEdge(v5);
        int a6 = cuboid.addEdge(v6);
        int a7 = cuboid.addEdge(v7);
        int a8 = cuboid.addEdge(v8);
        
        //texturcoordinaten indexieren:
        int linksu = cuboid.addTextureCoord(lu);
        int rechtsu = cuboid.addTextureCoord(ru);
        int rechtso = cuboid.addTextureCoord(ro);
        int linkso = cuboid.addTextureCoord(lo);
        
        int[] texcoords = {linksu, rechtsu, rechtso, linkso};
        
        //surfaces:
        Quad front = new Quad(cuboid, a1, a2, a3 ,a4); 
        Quad right = new Quad(cuboid, a2, a5, a8 ,a3); 
        Quad back = new Quad(cuboid, a5, a6, a7 ,a8); 
        Quad left = new Quad(cuboid, a6, a1 , a4 ,a7); 
        Quad top = new Quad(cuboid, a4, a3, a8 ,a7); 
        Quad bottom = new Quad(cuboid, a6, a5, a2 ,a1);

        //Texturkoordinaten setzen:
        for (int i=0; i<texcoords.length; i++){
            front.addTexCoord(texcoords[i]);
            right.addTexCoord(texcoords[i]);
            back.addTexCoord(texcoords[i]);
            left.addTexCoord(texcoords[i]);
            top.addTexCoord(texcoords[i]);
            bottom.addTexCoord(texcoords[i]);
        }
        
        //normalen:
        GLVector n1 = MathHelper.calcNormal(v1,v2,v3);
        GLVector n2 = MathHelper.calcNormal(v2,v5,v8);
        GLVector n3 = MathHelper.calcNormal(v5,v6,v7);
        GLVector n4 = MathHelper.calcNormal(v6,v1,v4);
        GLVector n5 = MathHelper.calcNormal(v4,v3,v8);
        GLVector n6 = MathHelper.calcNormal(v6,v5,v1);
        
        //normalen adden:
        if (cuboid.isRoundNormals()){
            int[] normplaces1 = {a1, a2, a3, a4};
            cuboid.addNormal(n1, normplaces1);
            front.addNormal(normplaces1);
            
            int[] normplaces2 = {a2, a5, a8, a3};
            cuboid.addNormal(n2, normplaces2);
            right.addNormal(normplaces2);
            
            int[] normplaces3 = {a5, a6, a7 ,a8};
            cuboid.addNormal(n3, normplaces3);
            back.addNormal(normplaces3);
            
            int[] normplaces4 = {a6, a1, a4 ,a7};
            cuboid.addNormal(n4, normplaces4);
            left.addNormal(normplaces4);
            
            int[] normplaces5 = {a4, a3, a8 ,a7};
            cuboid.addNormal(n5, normplaces5);
            top.addNormal(normplaces5);
            
            int[] normplaces6 = {a6, a5, a1 ,a2};
            cuboid.addNormal(n6, normplaces6);
            bottom.addNormal(normplaces6);
        }else {
            cuboid.addNormal(n1, new int[] {0});
            front.addNormal(new int[] {0});
            
            cuboid.addNormal(n2, new int[] {1});
            right.addNormal(new int[] {1});
            
            cuboid.addNormal(n3, new int[] {2});
            back.addNormal(new int[] {2});
            
            cuboid.addNormal(n4, new int[] {3});
            left.addNormal(new int[] {3});
            
            cuboid.addNormal(n5, new int[] {4});
            top.addNormal(new int[] {4});
            
            cuboid.addNormal(n6, new int[] {5});
            bottom.addNormal(new int[] {5});
        }
        
        
        //surfaces adden
        cuboid.addSurface(front);
        cuboid.addSurface(right);
        cuboid.addSurface(back);
        cuboid.addSurface(left);
        cuboid.addSurface(top);
        cuboid.addSurface(bottom);
        
        cuboid.autoTexCoords = false;
        return cuboid;
    }
    
    public static Mesh createPyramide(float length, float depth, float height){
        Vertex v1 = new Vertex(-(length / 2f), -height/2f, (depth/2f)); //vorne l
        Vertex v2 = new Vertex(-(length / 2f), -height/2f, -(depth/2f)); //hinten l
        Vertex v3 = new Vertex((length / 2f), -height/2f, -(depth/2f));
        Vertex v4 = new Vertex((length / 2f), -height/2f, (depth/2f));
        Vertex v5 = new Vertex(0f, height/2f, 0f);
        
        Mesh pyramide = new Mesh();
        //eckpunkte dem mesh hinzufgen:
        int a1 = pyramide.addEdge(v1);
        int a2 = pyramide.addEdge(v2);
        int a3 = pyramide.addEdge(v3);
        int a4 = pyramide.addEdge(v4);
        int a5 = pyramide.addEdge(v5);
        
        //texturcoordinaten indexieren:
        int linksu = pyramide.addTextureCoord(lu);
        int rechtsu = pyramide.addTextureCoord(ru);
        int obenmitte = pyramide.addTextureCoord(om);
        int linkso = pyramide.addTextureCoord(lo);
        int rechtso = pyramide.addTextureCoord(ro);
        
        int[] texcoords = {linksu, rechtsu, obenmitte,linkso, rechtso};

        
        //surfaces:
        Triangle front = new Triangle(pyramide, a1, a4, a5);
        Triangle right = new Triangle(pyramide, a4, a3, a5);
        Triangle back = new Triangle(pyramide, a3, a2, a5);
        Triangle left = new Triangle(pyramide, a2, a1, a5);
        Quad bottom = new Quad(pyramide, a2, a3, a4, a1);
        
        // normalen erstellen :
        GLVector n1 = MathHelper.calcNormal(v1, v4, v5); 
        GLVector n2 = MathHelper.calcNormal(v4, v3, v5);
        GLVector n3 = MathHelper.calcNormal(v3, v2, v5);
        GLVector n4 = MathHelper.calcNormal(v2, v1, v5);
        GLVector n5 = MathHelper.calcNormal(v2, v3, v4);
        
        //normalen hinzufgen (eckig):
        pyramide.addNormal(n1, new int[] {0});
        front.addNormal(new int[] {0});
        
        pyramide.addNormal(n2, new int[] {1});
        right.addNormal(new int[] {1});
        
        pyramide.addNormal(n3, new int[] {2});
        back.addNormal(new int[] {2});
        
        pyramide.addNormal(n4, new int[] {3});
        left.addNormal(new int[] {3});
       
        pyramide.addNormal(n5, new int[] {4});
        bottom.addNormal(new int[] {4});
        
        
        //Texturkoordinaten setzen:
        for (int i=0; i<3; i++){
            front.addTexCoord(texcoords[i]);
            right.addTexCoord(texcoords[i]);
            back.addTexCoord(texcoords[i]);
            left.addTexCoord(texcoords[i]);
            //bottom.addTexCoord(texcoords[i]);
        }
        bottom.addTexCoord(texcoords[0]);
        bottom.addTexCoord(texcoords[1]);
        bottom.addTexCoord(texcoords[3]);
        bottom.addTexCoord(texcoords[4]);
        
        
        // surfaces adden
        pyramide.addSurface(front);
        pyramide.addSurface(right);
        pyramide.addSurface(back);
        pyramide.addSurface(left);
        pyramide.addSurface(bottom);
        
        
        pyramide.autoTexCoords = false;
        return pyramide;
    }
    
    public static Mesh createCube(float size){
        return new Mesh(createCuboid(size, size, size));
    }
    
    public static Mesh createTrunk(){
        //Stamm Vertices erstellen 
        //TODO: beachten: absolute Zahlen evtl unschn!
        Vertex v1 = new Vertex(-4f, -8f, 3.5f); //unten vorne links
        Vertex v3 = new Vertex(4f, -8f, 3.5f); //unten vorne rechts
        Vertex v2 = new Vertex(0f, -8f, -3.5f); //unten hinten
        Vertex v4 = new Vertex(-2f, 8f, -1.75f); //oben hinten links
        Vertex v5 = new Vertex(0f, 8f, 1.75f); //oben vorne
        Vertex v6 = new Vertex(2f, 8f, -1.75f); // oben hinten rechts
        
        Mesh trunk = new Mesh();
       // trunk.deleteNormals();
       // trunk.setRoundNormals(true);
        
        //Eckpunkte hinzufgen:
        int a1 = trunk.addEdge(v1);
        int a2 = trunk.addEdge(v2);
        int a3 = trunk.addEdge(v3);
        int a4 = trunk.addEdge(v4);
        int a5 = trunk.addEdge(v5);
        int a6 = trunk.addEdge(v6);
        
        //Surfaces erstellen:
        Triangle front = new Triangle(trunk, a1, a3, a5);
        Triangle right = new Triangle(trunk, a3, a6, a5);
        Triangle back = new Triangle(trunk, a6, a2, a4);
        Triangle left = new Triangle(trunk, a4, a1, a5);
        Triangle top = new Triangle(trunk, a4, a5, a6);
        Triangle bottom = new Triangle(trunk, a1, a2, a3);
        Triangle rightBack = new Triangle(trunk, a6, a3, a2);
        Triangle leftBack = new Triangle(trunk, a4, a2, a1);
        
        // normalen erstellen :
        GLVector n1 = MathHelper.calcNormal(v1, v3, v5); 
        GLVector n2 = MathHelper.calcNormal(v3, v6, v5);
        GLVector n3 = MathHelper.calcNormal(v6, v2, v4);
        GLVector n4 = MathHelper.calcNormal(v4, v1, v5);
        GLVector n5 = MathHelper.calcNormal(v4, v5, v6);
        GLVector n6 = MathHelper.calcNormal(v1, v2, v3);
        GLVector n7 = MathHelper.calcNormal(v6, v3, v2);
        GLVector n8 = MathHelper.calcNormal(v4, v2, v1);
        
        
        // normalen adden:
        if (trunk.isRoundNormals()){
            int[] normplaces1 = {a1, a3, a5};
            trunk.addNormal(n1, normplaces1);
            front.addNormal(normplaces1);
            
            int[] normplaces2 = {a3, a6, a5};
            trunk.addNormal(n2, normplaces2);
            right.addNormal(normplaces2);
            
            int[] normplaces3 = {a6, a2 ,a4};
            trunk.addNormal(n3, normplaces3);
            back.addNormal(normplaces3);
            
            int[] normplaces4 = {a4, a1, a5};
            trunk.addNormal(n4, normplaces4);
            left.addNormal(normplaces4);
            
            int[] normplaces5 = {a4, a5, a6};
            trunk.addNormal(n5, normplaces5);
            top.addNormal(normplaces5);
            
            int[] normplaces6 = {a1, a2, a3};
            trunk.addNormal(n6, normplaces6);
            bottom.addNormal(normplaces6);
            
            int[] normplaces7 = {a6, a2, a3};
            trunk.addNormal(n5, normplaces7);
            top.addNormal(normplaces7);
            
            int[] normplaces8 = {a4, a2, a1};
            trunk.addNormal(n6, normplaces8);
            bottom.addNormal(normplaces8);
        }else {
            trunk.addNormal(n1, new int[] {0});
            front.addNormal(new int[] {0});
            
            trunk.addNormal(n2, new int[] {1});
            right.addNormal(new int[] {1});
            
            trunk.addNormal(n3, new int[] {2});
            back.addNormal(new int[] {2});
            
            trunk.addNormal(n4, new int[] {3});
            left.addNormal(new int[] {3});
            
            trunk.addNormal(n5, new int[] {4});
            top.addNormal(new int[] {4});
            
            trunk.addNormal(n6, new int[] {5});
            bottom.addNormal(new int[] {5});
            
            trunk.addNormal(n7, new int[] {6});
            rightBack.addNormal(new int[] {6});
            
            trunk.addNormal(n8, new int[] {7});
            leftBack.addNormal(new int[] {7});
        }
        
        //surfaces adden:
        trunk.addSurface(front);
        trunk.addSurface(right);
        trunk.addSurface(back);
        trunk.addSurface(left);
        trunk.addSurface(top);
        trunk.addSurface(bottom);
        
        trunk.addSurface(rightBack);
        trunk.addSurface(leftBack);
        
        return trunk;
    }
    
    public static Mesh createPlateQuads(float length){
        Vertex v1 = new Vertex(-length/2, -length/2, 0f); //vorne unten-l
        Vertex v2 = new Vertex(length/2, -length/2, 0f); //vorne u-r
        Vertex v3 = new Vertex(length/2, length/2, 0f); //vorne oben-r
        Vertex v4 = new Vertex(-length/2, length/2, 0f); //vorne oben-l
        
        Mesh plate = new Mesh();
        int a = plate.addEdge(v1);
        int b = plate.addEdge(v2);
        int c = plate.addEdge(v3);
        int d = plate.addEdge(v4);
        
        //Normalen
        GLVector n1 = MathHelper.calcNormal(v1, v2, v3); 
        plate.addNormal(n1, new int[] {0});
        
        //texture koords indexieren:
        int linksu = plate.addTextureCoord(lu);
        int rechtsu = plate.addTextureCoord(ru);
        int rechtso = plate.addTextureCoord(ro);
        int linkso = plate.addTextureCoord(lo);
        
        Quad front = new Quad(plate, a, b, c, d);
        front.addNormal( new int[] {0});
        
        //texcoords setzen:
        front.addTexCoord(linksu);
        front.addTexCoord(rechtsu);
        front.addTexCoord(rechtso);
        front.addTexCoord(linkso);
        
        plate.addSurface(front);
        plate.autoTexCoords = false;
        return plate;
    }
   
}
