/*
 * Created on 28.07.2006
 *
 */
package big3d.objects;

import java.util.ArrayList;

import big3d.camera.Camera;
import big3d.material.Material;
import big3d.material.TextureLib;
import big3d.material.TextureObject;
import big3d.utils.Color;
import big3d.utils.GLSettings;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;

public class SkyBox {
    private static SkyBox instance = null;
    private int displayListNumber = -1;
    private float size = 5f;
    private ArrayList<Material> textures= null;
    
    public static SkyBox getInstance(){
        if (instance == null)
            instance = new SkyBox();
        return instance;
    }
    
    private SkyBox(){
        textures = new ArrayList<Material>();
        
        //Material laden:
        Material front = new Material();
        Material right = new Material();
        Material back = new Material();
        Material left = new Material();
        Material top = new Material();
        Material bottom = new Material();
        
        //Texturen zuordnen:
        front.addTexture(TextureLib.getInstance().getTextureID(new TextureObject("src/big3d/textures/skys/night/front.jpg")));
        right.addTexture(TextureLib.getInstance().getTextureID(new TextureObject("src/big3d/textures/skys/night/right.jpg")));
        back.addTexture(TextureLib.getInstance().getTextureID(new TextureObject("src/big3d/textures/skys/night/back.jpg")));
        left.addTexture(TextureLib.getInstance().getTextureID(new TextureObject("src/big3d/textures/skys/night/left.jpg")));
        top.addTexture(TextureLib.getInstance().getTextureID(new TextureObject("src/big3d/textures/skys/night/top.jpg")));
        bottom.addTexture(TextureLib.getInstance().getTextureID(new TextureObject("src/big3d/md2/players/holz.jpg")));
        
        //Material merken:
        textures.add(front);
        textures.add(right);
        textures.add(back);
        textures.add(left);
        textures.add(top);
        textures.add(bottom);
    }
    
    private void createSkyList(GLDrawable gld){
        GL gl = gld.getGL();
        
        float side = this.size  / 2;
        
        this.displayListNumber = gl.glGenLists(1);
        gl.glNewList(this.displayListNumber, GL.GL_COMPILE);
        
        gl.glDisable(GL.GL_LIGHTING);
        
        gl.glEnable(GL.GL_TEXTURE_2D);
        
        GLSettings.setDefaultTextureEnv(gl); 
        // Texturumgebeung setzen:
        gl.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_MODULATE);
       
        GLSettings.setTextureParamClamped(gl);
        
        //FRONT
        gl.glBindTexture(GL.GL_TEXTURE_2D, TextureLib.getInstance().
                getTexture(this.textures.get(0).getTextures().get(0)).getMyOpenGLID());
        gl.glBegin(GL.GL_QUADS);
        
        gl.glNormal3f(0.0f, 0.0f, -1.0f);
        
        gl.glTexCoord2f(0.0f, 0.0f); 
        gl.glVertex3f(-side, -side, -side); //1
        gl.glTexCoord2f(0.0f, 1.0f); 
        gl.glVertex3f(-side,  side, -side);  //2
       
        gl.glTexCoord2f(1.0f, 1.0f);
        gl.glVertex3f( side,  side, -side); //3
        gl.glTexCoord2f(1.0f, 0.0f);
        gl.glVertex3f( side, -side, -side); //4
       
        gl.glEnd();
        
        //RIGHT
        gl.glBindTexture(GL.GL_TEXTURE_2D, TextureLib.getInstance().
                getTexture(this.textures.get(1).getTextures().get(0)).getMyOpenGLID());
        gl.glBegin(GL.GL_QUADS);
       
        gl.glNormal3f(1.0f, 0.0f, 0.0f);
        gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f( side, -side, -side);
        gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f( side,  side, -side);
        gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f( side,  side,  side);
        gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f( side, -side,  side);
        gl.glEnd();
        
        //BACK
        gl.glBindTexture(GL.GL_TEXTURE_2D, TextureLib.getInstance().
                getTexture(this.textures.get(2).getTextures().get(0)).getMyOpenGLID());
        
        gl.glBegin(GL.GL_QUADS);
        gl.glNormal3f(0.0f, 0.0f, 1.0f);
        gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(-side, -side,  side);
        gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f( side, -side,  side);
        gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f( side,  side,  side);
        gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(-side,  side,  side);
       
        gl.glEnd();
        
        //LEFT
        gl.glBindTexture(GL.GL_TEXTURE_2D, TextureLib.getInstance().
                getTexture(this.textures.get(3).getTextures().get(0)).getMyOpenGLID());
        gl.glBegin(GL.GL_QUADS);
 
        gl.glNormal3f(-1.0f, 0.0f, 0.0f);
        gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(-side, -side, -side);
        gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(-side, -side,  side);
        gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(-side,  side,  side);
        gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(-side,  side, -side);
        gl.glEnd();
        
        //TOP
        gl.glBindTexture(GL.GL_TEXTURE_2D, TextureLib.getInstance().
                getTexture(this.textures.get(4).getTextures().get(0)).getMyOpenGLID());
        gl.glBegin(GL.GL_QUADS);
       
        gl.glNormal3f(0.0f, 1.0f, 0.0f);
        gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(-side, side, -side);
        gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(-side,  side,  side);
        gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f( side,  side,  side);
        gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f( side,  side, -side);
        gl.glEnd();
        
        //BOTTOM
         gl.glBindTexture(GL.GL_TEXTURE_2D, TextureLib.getInstance().
                getTexture(this.textures.get(5).getTextures().get(0)).getMyOpenGLID());
         gl.glBegin(GL.GL_QUADS);
        
         gl.glNormal3f(0.0f, -1.0f, 0.0f);
         gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(-side, -side, -side);
         gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f( side, -side, -side);
         gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f( side, -side,  side);
         gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(-side, -side,  side);
         gl.glEnd();
         gl.glEnable(GL.GL_LIGHTING);
        gl.glEndList();

    }
    
    
    public void draw(GLDrawable gld, Camera cam){
        GL gl =gld.getGL();
        
        //Falls die DisplayListe noch nicht erstellt wurde:
        if (this.displayListNumber == -1)
            this.createSkyList(gld); 

        gl.glDisable(GL.GL_DEPTH_TEST);
        gl.glDepthMask(false);
        
        gl.glPushMatrix();
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity(); 

        cam.camTransform.composeGLMatrixNoTransformation(); 
        
        gl.glLoadMatrixf(cam.camTransform.m_matrix);
        
        gl.glCallList(this.displayListNumber);
       
        gl.glPopMatrix();
        
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthMask(true);
        
   }

}
