/*
 * Created on 08.05.2006
 *
 */
package big3d.utils;

import net.java.games.jogl.GL;

public class MathHelper {
//  Minimum Double = 4.9E-324, Minimum Float = 1.4E-45 
    // double-Zahlen kleiner als GRENZWERT sollen als Null interpretiert werden
    private final static double GRENZWERT = 1e-300;     
    
    /**
     * Ermittelt die Lnge eines Vektors
     * 
     * @param vector
     * @return
     */
    public static float vectorLength(GLVector vector){
        float length = 0f;
        
        //bilde die quadrate und addiere:
        length = vector.getX()*vector.getX() + vector.getY()*vector.getY() +
            vector.getZ()*vector.getZ();
        
        //ziehe die wurzel:
        length = (float) Math.sqrt(length);
        
        //fertig
        return length;
    }
    
    /**
     * Addiert zwei Vektoren und liefert den Ergebnisvektor
     * 
     * @param v1
     * @param v2
     * @return
     */
    public static GLVector vectorAddVector(GLVector v1, GLVector v2){
        return new GLVector(v1.getX() + v2.getX(),
                v1.getY() + v2.getY(),
                v1.getZ() + v2.getZ());
    }
    
    /**
     * Multipliziert einen Vektor mit einem Skalar
     * 
     * @param v
     * @param s
     * @return
     */
    public static GLVector vectorMultSkalar(GLVector v, float s){
        return new GLVector(s*v.getX(), s*v.getY(), s*v.getZ());
    }
    
    /**
     * Berechnet das kartesische Produkt zweier Vektoren
     * @param v1
     * @param v2
     * @return
     */
    public static GLVector vectorKartProdVector(GLVector v1, GLVector v2){
        return new GLVector(
                v1.getY() * v2.getZ() - v1.getZ() * v2.getY(),
                v1.getZ() * v2.getX() - v1.getX() * v2.getZ(),
                v1.getX() * v2.getY() - v1.getY() * v2.getX()
        );
    }
    
    /**
     * Berechnet den Normalenvektor zu einer Ebene. Diese wird durch drei Punkte
     * angegeben.
     * @param v1
     * @param v2
     * @param v3
     * @return
     */
    public static GLVector calcNormal(Vertex v1, Vertex v2, Vertex v3){
        //berechne die Sttz-Vektoren der Ebene:
        GLVector vec1 = createVectorFromVertices(v1,v2);
        GLVector vec2 = createVectorFromVertices(v1,v3);
        
        //berechne Kreuzprodukt:
        GLVector nearlyResult = vectorKartProdVector(vec1,vec2);
        
        //normalisieren und zurckgeben:
        return normalizeVector(nearlyResult);
    }
    
    /**
     * Normalisiert einen Vector
     * @param v
     * @return
     */
    public static GLVector normalizeVector(GLVector v){
        float length = vectorLength(v); 
        
        //normalisieren und zurckgeben:
        return new GLVector(
                v.getX()/length,
                v.getY()/length,
                v.getZ()/length);
    }
    
    /**
     * Berechnen einen Vector, der von einem Punkt zu einem anderen zeigt.
     * @param from
     * @param to
     * @return
     */
    public static GLVector createVectorFromVertices(Vertex from, Vertex to){
        return new GLVector(
                to.getX() - from.getX(),
                to.getY() - from.getY(),
                to.getZ() - from.getZ()
        );
    }
    
    /**
     * Testet, ob zwei Vektoren gleich sind. Diese mssen gleich lang sein!
     * @param v1
     * @param v2
     * @return
     */
    public static boolean vectorEqualsVector(GLVector v1, GLVector v2){
        if (v1.getX() != v2.getX() || v1.getY() != v2.getY() || v1.getZ() != v2.getZ())
            return false;
        return true;
    }
    
    
    public static void rotate(GL gl, float xAchse, float yAchse, float zAchse){
        gl.glRotatef(xAchse,1,0,0);
        gl.glRotatef(yAchse,0,1,0);
        gl.glRotatef(zAchse,0,0,1);
    }
    
    /**
     * Multipliziert zwei transponierte Matrizen miteinander.
     * 
     * @param transA die erste transponierte Matrix
     * @param transB die zweite transponierte Matrix
     * @return das Ergebnis der Multiplikation
     */
    public static float[] mTransMultTrans(float[] transA, float[] transB) {
        float[] temp = new float[16];
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                for (int k = 0; k <4; k++) {
                    temp[i + (j*4)] += transA[i+(k*4)] * transB[k+(j*4)];
                }

            }
        }
        return temp;
    }   
    
    /**
     * Berechnet das Ergebnis eines linearen Gleichungssystems
     * zwischen zwei Matrizen.
     * 
     * @param a die erste Matrix
     * @param b die zweite Matrix
     * @return Liefert die L&ouml;sung des LGS.
     */
    public static float[][] gauss(float[][] a, float[] b) {
        
        // a und b sollten gleichviele Zeilen enthalten
        if (b.length != a.length) return (float[][]) null;
        
        // leere LGS-Matrix anlegen:
        //   Anzahl Zeilen = Anzahl der Gleichungen
        //   Anzahl Spalten = Anzahl der Variablen + 1 fr b
        double[][] lgs = new double[a.length][a[0].length + 1];
        
        // Kopiere a mit angehaengtem b in die neue Matrix lgs
        for(int i = 0; i < a.length; i++) {
            
            // ungleichmaessige Eingabe-Matrix a -> ungueltig, daher Abbruch
            if (a[i].length > a[0].length) return (float[][]) null;
            
            for(int j = 0; j < a[i].length; j++) {
                lgs[i][j] = a[i][j];
            }
            
            lgs[i][lgs[0].length - 1] = b[i];   
        }

        return berechne(lgs);
    }
    
    /**
     * Eigentliche Brechnung des LGS.
     * 
     * <p>Es wird dabei direkt auf der kopierten lgs-Matrix gearbeitet</p>
     * 
     * @param lgs die Matrix
     * @return Liefert die L&ouml;sung des LGS
     */
    private static float[][] berechne(double[][] lgs) {
        int m = lgs.length;         // Anzahl Zeilen bzw. Gleichungen
        int n = lgs[0].length - 1;      // Anzahl Spalten ohne b = Anzahl Variablen 
        int tempZeile;              // temp-Variable
        int anzahlInStufenform = 0; // Anzahlen der Zeilen in Stufenform
        
        int anzahlMaximalBerechenbarerVariablen = Math.min(m, n);

        // Da ein Spaltentausch im Gegensatz zum Zeilentausch spaeter wieder 
        //  rueckgaengig gemacht werden muss, wird in diesem Vektor gespeichert, 
        //  in welcher neuen Spalte 'ns' sich die urspruengliche Spalte 'us'
        //  befindet:
        //      getauschteSpalten[us] = ns 
        //      <=> 
        //      Spalte us nach der Berechnung in Spalte ns liegt.
        int[] getauschteSpalten = new int[n];
        for(int j = 0; j < n; j++) {
            getauschteSpalten[j] = j;
        }
        

        // Stufenform erzeugen, dazu die einzelen Spalten bearbeiten.
        // Ziel: (* = irgendwas)
        //      / 1 * *   * * *   * \
        //      | 0 1 *   * * *   * |
        //      | 0 0 1   * * *   * |
        //      |                   |
        //      | 0 0 0   0 0 0   * |
        //      \ 0 0 0   0 0 0   * /
    
        StufenformErzeugen:
        for(int j = 0; j < anzahlMaximalBerechenbarerVariablen; j++) {
            
            tempZeile = sucheZeileDieserSpalteUngleichNull(j, j, lgs, m, n);

            // tempZeile == -1 <=> es wurde keine Zeile gefunden.
            if (tempZeile == -1) {
                
                // nach geeigneter Zeile suchen - mit Spaltentausch.
                tempZeile = sucheZeileMitSpalteUngleichNullUndTauscheSpalten(
                        j, lgs, getauschteSpalten, m, n);
                
                // immer noch nichts gefunden? dann ist die dreiecksform fertig
                if (tempZeile == -1) {
                    break StufenformErzeugen;
                }
            }

            // bringe die gefundene Zeile nach oben, falls erforderlich
            tauscheZeilen(tempZeile, j, lgs);
            
            // fuer die Dreicksform: diese Spalte in allen tieferen Zeilen
            //  auf Null bringen.
            restlicheZeilenDieserSpalteAufNullBringen(j, lgs, m, n);
            
            anzahlInStufenform++;
        }

        // ObereDreiecksform erzeugen. Ziel:
        // Ziel: (* = irgendwas)
        //      / 1 0 0   * * *   * \
        //      | 0 1 0   * * *   * |
        //      | 0 0 1   * * *   * |
        //      |                   |
        //      | 0 0 0   0 0 0   * |
        //      \ 0 0 0   0 0 0   * /

        if (anzahlInStufenform > 1) {
            
            // t = Zeile/Spalte, die abgezogen wird
            for(int t = (anzahlInStufenform - 1); t > 0; t--) {
                
                // bringe Spalte t in allen Zeilen i < t auf Null 
                for(int i = 0; i < t; i++) {
                    for(int j = (t+1); j < (n+1); j++) {
                        
                        lgs[i][j] = lgs[i][j] - (lgs[i][t] * lgs[t][j]); 
                        
                        // ist der Wert zu nah an Null, wird angenommen, 
                        //  dass er gleich Null ist. Nicht ungefaehrlich.
                        if (Math.abs(lgs[i][j]) < GRENZWERT) lgs[i][j] = 0.0;

                    }
                    lgs[i][t] = 0.0;
                }
            }
        }
        
        return bildeLoesung(lgs, anzahlInStufenform, getauschteSpalten, m, n);
    }
    
    /**
     * Berechnet die endg&uuml;ltige L&ouml;sung des LGS.
     * 
     * @param lgs die LGS-Matrix
     * @param anzahlInStufenform die Anzahl der Stufen
     * @param getauschteSpalten die getauschten Spalten
     * @param m Anzahl Spalten der Matrix
     * @param n Anzahl Zeilen der Matrix
     * @return die L&ouml;sungsmatrix
     */
    private static float[][] bildeLoesung(double[][] lgs, 
            int anzahlInStufenform, int[] getauschteSpalten, int m, int n) {

        float[][] l;
        int tempCounter;
        
        // Fall 1: unloesbar: es gibt (unten) zeilen der Form:
        //  <0 ... 0   * > mit * ungleich 0 -> LGS ist unloesbar.
        if (anzahlInStufenform < m) {
            for(int i = anzahlInStufenform; i < m; i++) {
                System.out.println(lgs[i][n] != 0.0);
                if (lgs[i][n] != 0.0) return (float[][]) null;
            }
        }
        
        // Fall 2: eindeutig loesbar:
        if (anzahlInStufenform == n) {
            l = new float[n][1];
            
            for(int i = 0; i < n; i++) {
                l[getauschteSpalten[i]][0] = (float) lgs[i][n];
            }
            return l;
        }
        
        // Fall 3: mehrdeutig loesbar: Konstruieret Loesungsraum
        //  Anzahl Vektoren = 1 (Basis) + Anzahl der nicht eindeutig geloesten 
        //  Variablen. 
        l = new float[n][n - anzahlInStufenform + 1];
        for(int i = 0; i < anzahlInStufenform; i++) {
            l[getauschteSpalten[i]][0] = (float) lgs[i][n];
            
            for(int j = 0; j < (n - anzahlInStufenform); j++) {
                l[getauschteSpalten[i]][j + 1] = 
                    -1.0f * (float) lgs[i][anzahlInStufenform + j];
            }
        }
        
        tempCounter = 1;
        for(int i = anzahlInStufenform; i < n; i++) {
            l[getauschteSpalten[i]][tempCounter] = 1.0f;
            tempCounter++;
        }
        
        return l;
    }
    
    /**
     * Tauscht zwei Zeilen miteinander in der Matrix.
     * 
     * <p>Da Arrays Objekte sind (-> Zeiger), ist das Tauschen sehr einfach.</p>
     * @param z1 Zeile eins
     * @param z2 Zeile 2
     * @param lgs LGS-Matrix
     */
    private static void tauscheZeilen(int z1, int z2, double[][] lgs) {
        if (z1 != z2) {
            double[] tempZeile = lgs[z1];
            lgs[z1] = lgs[z2];
            lgs[z2] = tempZeile;
        }
    }

    /**
     * Tauscht zwei Spalten miteinander in der Matrix.
     * 
     * @param s1 die erste Spalte
     * @param s2 die zweite Spalte
     * @param lgs die Matrix
     * @param getauschteSpalten die getauschten Spalten
     * @param m Anzahl Spalten der Matrix
     */
    private static void tauscheSpalten(
            int s1, int s2, double[][] lgs, int[] getauschteSpalten, int m) {
        
        // eintraege in der getauschteSpalten-Matrix updaten
        int tempSpaltenNummer = getauschteSpalten[s1];
        getauschteSpalten[s1] = getauschteSpalten[s2];
        getauschteSpalten[s2] = tempSpaltenNummer;
        
        // die Spalten im lgs tauschen
        double temp;
        for (int i = 0; i < m; i++) {
            temp = lgs[i][s1];
            lgs[i][s1] = lgs[i][s2];
            lgs[i][s2] = temp;
        }
    }
    
    /**
     * Sucht eine Zeile, die keine Null-Zeile ist.
     * 
     * <p>Das geschieht in der Matrix wie folgt:<br>
     * Es wird eine Zeile unterhalb oder gleich sp gesucht, dessen Spalte sp 
     * ungleich Null ist.<br>
     * <b>Nicht:</b> suchen nach gr&ouml;&szlig;ter Zahl o.&Auml;.</p>
     * 
     * @param zeile - Counter der Zeile
     * @param sp - Counter der Spalte
     * @param lgs - die LGS-Matrix
     * @param m - Anzahl der Spalten der Matrix
     * @param n - Anzahl der Zeilen der Matrix
     * @return Liefert die erste gefundene Zeilenzahl i. Ist per Default auf -1 gesetzt.
     */
    private static int sucheZeileDieserSpalteUngleichNull(
            int zeile, int sp, double[][] lgs, int m, int n) {
        
        for(int i = zeile; i < m; i++) {
            if (lgs[i][sp] != 0.0) return i;
        }
        return -1;
    }

    

    /**
     * Sucht eine Spalte, die keine Null-Spalte ist.
     * 
     * <p>Das geschieht in der Matrix wie folgt:<br> 
     * Sucht in allen tieferen Zeilen nach einem Spalteneintrag ungleich Null.
     * Wird er f&uuml;ndig, werden die Spalten getauscht.</p>
     * 
     * @param sp - Spalte
     * @param lgs - die LGS-Matrix
     * @param getauschteSpalten - Speicherung der getauschten Spalten
     * @param m - Anzahl der Spalten der Matrix
     * @param n - Anzahl der Zeilen der Matrix
     * @return Liefert das Ergebnis der Suche, ist per Default auf -1 gesetzt.
     */
    private static int sucheZeileMitSpalteUngleichNullUndTauscheSpalten(
            int sp, double[][] lgs, int[] getauschteSpalten, int m, int n) {
        
        int tempZeile;
        
        for(int j = (sp + 1); j < n; j++) {
            tempZeile = sucheZeileDieserSpalteUngleichNull(sp, j, lgs, m, n);
            if (tempZeile != -1) {
                tauscheSpalten(j, sp, lgs, getauschteSpalten, m);
                return tempZeile;
            }
        }
        return -1;
    }
    
    
    /**
     * Erstellt aus verbleibenden Zeilen Null-Zeilen.
     * 
     * @param sp Start-Zeile und -Spalte
     * @param lgs die LGS-Matrix
     * @param m Anzahl der Spalten der Matrix
     * @param n Anzahl der Zeilen der Matrix
     */
    private static void restlicheZeilenDieserSpalteAufNullBringen(
            int sp, double[][] lgs, int m, int n) {

        // Frage: Ist es wirklich sinnvoll, den Eintrag lgs[sp][sp] auf 1 zu 
        //  bringen, oder erhoeht sich dadurch noch der Rundungsfehler?
        if (lgs[sp][sp] != 1.0) {
            for(int j = (sp + 1); j < (n + 1); j++) {
                lgs[sp][j] = lgs[sp][j] / lgs[sp][sp];
            }
            lgs[sp][sp] = 1.0;
        }

        // alle folgenden Zeilen in dieser Spalte auf Null bringen
        for(int i = (sp + 1); i < m; i++) {
            
            // Nur Eintraege ubgleich Null muessen auf Null gebracht werden.
            if (lgs[i][sp] != 0.0) {
                for(int j = (sp + 1); j < (n + 1); j++) {
                    
                    // vorsicht rundungsfehler!
                    lgs[i][j] = lgs[i][j] - (lgs[i][sp] * lgs[sp][j]);
                    
                    // ist der Wert zu nah an Null, wird angenommen, 
                    //  dass er gleich Null ist. Nicht ungefaehrlich.
                    if (Math.abs(lgs[i][j]) < GRENZWERT) lgs[i][j] = 0.0;
                }
                lgs[i][sp] = 0.0;
            }
        }
    }
}
