/*
 * Created on 30.05.2006
 *
 */
package big3d.worlds;

import java.util.ArrayList;

import big3d.camera.Camera;
import big3d.collision.CollisionComputer;
import big3d.listener.CanvasListener;
import big3d.material.TextureLib;
import big3d.objects.FogObject;
import big3d.objects.LightFactory;
import big3d.objects.LightObject;
import big3d.objects.Mesh;
import big3d.objects.SkyBox;
import big3d.objects.partikel.Partikel;
import big3d.objects.RotatingObject;
import big3d.sound.Soundengine;
import big3d.utils.GLSettings;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;
import net.java.games.jogl.GLU;
import net.java.games.jogl.util.GLUT;

public abstract class AbstractWorld {
    
    protected ArrayList<Integer> lights;
    protected ArrayList<Integer> solidObjects;
    protected ArrayList<Mesh> movableObjects;
    protected ArrayList<Integer> transparentObjects;
    protected SkyBox box = null;
    protected FogObject fog = null;
    protected ArrayList<Partikel> partikel;
    protected ArrayList<RotatingObject> rotatingGLUTObjects;
   
    protected CollisionComputer collComp;
    private Soundengine se = Soundengine.getInstance();
    
    public static float distanz= -100f;
    public static int list;
    protected Camera cam;
    protected int width = 0;
    protected int height = 0;
    
    protected final float NEARPLANE = 1f; 
    protected final float FARPLANE = 400f;
    
    boolean ortho = false;
    boolean needsUpdate = false;
    
    //Dieses muss von allen Kinder-Welten berschrieben werden! 
    public static AbstractWorld getInstance(){
        return null;
    }
    
    //Zum erstellen der DisplayListen os
    public void init(GLDrawable gld){
        GL gl = gld.getGL();
        lights = new ArrayList<Integer>();
        solidObjects = new ArrayList<Integer>();
        transparentObjects = new ArrayList<Integer>();
        movableObjects = new ArrayList<Mesh>();
        partikel = new ArrayList<Partikel>();
        rotatingGLUTObjects = new ArrayList<RotatingObject>();
       
        //Camera intialisieren: (WICHTIG vor adden des Listeners!)
        cam = Camera.getInstance(gld);
        
        GLSettings.init(gl);
        
        collComp = new CollisionComputer(cam);
        
        // setzt einen KeyboardListener auf diese Instanz
        CanvasListener listener = new CanvasListener(cam);
        gld.addKeyListener(listener);
        gld.addMouseListener(listener);
        gld.addMouseMotionListener(listener);
        gld.addMouseWheelListener(listener);
        GLSettings.initLight(gl);
    }
    
    //Zeichenmethode
    public void draw(GLDrawable gld){
        GL gl = gld.getGL();
        TextureLib.getInstance().initAll(gld);
        
        //TODO: wenn sinnvoll
        if (this.needsUpdate)
            updateWorldSettings();
        
        //Kamera setzen:
        cam.applyFrameTransform();
        
        //SkyBox zeichnen:
        if (this.box != null)
            box.draw(gld,cam);
        
        // Nebel einschalten
        if (this.fog != null)
            fog.draw(gl);
        

        // schalte alle lichter an:
        for (int i=0; i<lights.size(); i++){
            LightObject light = LightFactory.getLight(i);
            
            light.turnOnLight(gl);
            light.draw(gld);
        }
           
        // Zeichne alle soliden Objekte:
        for (int i=0; i<solidObjects.size(); i++){
            gl.glCallList(solidObjects.get(i));
        }

        // Zeichne alle soliden beweglichen Objekte:
        for (int i=0; i<movableObjects.size(); i++){
            movableObjects.get(i).draw(gld);
        }

        //zeichne die partikelsysteme:
        for (int i=0; i<partikel.size(); i++){
            partikel.get(i).draw(gl);
        }
        
        // Zeichne RotatingGLUTObjects
        for (int i=0; i<rotatingGLUTObjects.size(); i++){
           rotatingGLUTObjects.get(i).draw();   
        }
        
        // Zeichne alle nichtSoliden Objekte:
        for (int i=0; i<transparentObjects.size(); i++){
           // GLSettings.setBlendingOn(gl);
            gl.glCallList(transparentObjects.get(i));
         //   GLSettings.setBlendingOff(gl); //FIXME: sollte gehen, geht aber nur ohne^^
        }
        
        // Starte Kollisionstest
        collComp.testCollision();
        
        //RocknSound!
        se.play();
    }

    /**
     * @return Returns the ortho.
     */
    public boolean isOrtho() {
        return ortho;
    }

    /**
     * @param ortho The ortho to set.
     */
    public void setOrtho(boolean ortho) {
        this.ortho = ortho;
        this.needsUpdate = true;
    }
    
    public void updateWorldSettings(){
        //TODO: Ortho bzw Projection setzen!
    }
    
    public void reshape(GLDrawable gld, int x, int y, int width, int height){
        GL gl= gld.getGL();
        GLU glu = gld.getGLU();
        
        // Viewport und Projektionsmatrix anpassen          
        gl.glViewport(x, y, width, height);     
        
        this.height = height;
        this.width = width;
        
        if (!ortho){
            gl.glMatrixMode(GL.GL_PROJECTION );  
            gl.glLoadIdentity(); 
            
            float ratio = width/height;
            
            glu.gluPerspective(60.0f, ratio, NEARPLANE, FARPLANE);
            
            // In den Modelview Matrix -Modus wechseln
            gl.glMatrixMode(GL.GL_MODELVIEW);
        } else {
            //Otho ansicht
            GLSettings.setOrtho(gld, height, width);
            
            gl.glMatrixMode(GL.GL_MODELVIEW);
        } 
    }
    
    public void toggleOrtho(){
        if (ortho)
            ortho = false;
        else ortho = true;
        needsUpdate = true;
    }
    
    
}
