package big3d.worlds;

import java.util.ArrayList;

import big3d.material.Material;
import big3d.objects.DisplayListFactory;
import big3d.objects.LightFactory;
import big3d.objects.LightObject;
import big3d.objects.Mesh;
import big3d.objects.MeshFactory;
import big3d.utils.Color;
import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable;

public class StreetFactory extends AbstractWorld {
    
	static int aHouse = 0;
	private static StreetFactory instance = null;
	
    public static StreetFactory getInstance(){
        if (instance == null)
            instance = new StreetFactory();
        return instance;
    }
	
	public void init(GLDrawable gld) {
		super.init(gld);
		
		GL gl = gld.getGL();
		//		 Licht:
        this.lights = new ArrayList<Integer>();

        int id = LightFactory.createNewLight();
        LightObject light = LightFactory.getLight(id);
        light.setPosition4f(0f,30f,-20f, 1f);
        light.turnOnLight(gl);
        
        this.lights.add(id);
        
        CreateStreet(gld,20);
	}
	
	

	
	protected static Mesh[] Strasse(GLDrawable gld,boolean tunnel, float x, float y, float z) {
        GL gl = gld.getGL();
        
        // Mesh-Array erstellen 
        Mesh[] strasse = new Mesh[3];
        
        // Strassenbelag erzeugen
        Material strassenbelag = new Material();
        strassenbelag.setColor(new Color(0.7f,0.7f,0.7f,0.5f));
        strassenbelag.setAmbientAndDiffuse(new Color(1.0f, 0.3f, 0.3f, 0.5f));
        
        // Strassenteile erzeugen und bergeben
		Mesh belag = new Mesh(MeshFactory.createCuboid(1f,25f,70));
 		belag.setPosition(0f+x,-10f+y,0f+z);
 		belag.setMaterial(strassenbelag);
		strasse[0] = belag;
		
		Mesh belag2 = new Mesh(MeshFactory.createCuboid(1f,25f,70));
		belag2.setMaterial(strassenbelag);
		belag2.setPosition(30f+x,-10f+y,0f+z);
		strasse[1] = belag2;
		
		// Mittelstreifen
        Material mittelstreifen = new Material();
        mittelstreifen.setColor(new Color(1.0f, 1.0f, 1.0f, 1f));
        mittelstreifen.setAmbientAndDiffuse(new Color(1.0f, 0.3f, 0.3f, 0.5f));
        
		Mesh belagMitte2 = new Mesh(MeshFactory.createCuboid(1f,5f,70f));
		belagMitte2.setMaterial(mittelstreifen);
		belagMitte2.setPosition(15f+x,-10f+y,0f+z);
		strasse[2] = belagMitte2;
		
		if (tunnel) {
			return addTunnel(gld,strasse,x,y,z);
		} else {
			return strasse;
		}
	}
	
	public static Mesh[] addTunnel(GLDrawable gld, Mesh[] strasse, float x, float y, float z) {
        GL gl = gld.getGL();
        Mesh[] tunnelstrasse = new Mesh[6];
        for(int i = 0; i < strasse.length;i++){
        	tunnelstrasse[i] = strasse[i];
        }
        
        Material tunnel = new Material();
        tunnel.setColor(new Color(0.9f, 0.2f, 0.2f, 0.5f));
        tunnel.setAmbientAndDiffuse(new Color(1.0f, 0.3f, 0.3f, 0.5f));
        
		Mesh wand = new Mesh(MeshFactory.createCuboid(40,10f,70));
		wand.setPosition(-17.5f+x,9.5f+y,0f+z);
 		wand.setMaterial(tunnel);
 		
		tunnelstrasse[3] = wand;
		
		Mesh wand2 = new Mesh(MeshFactory.createCuboid(40,10f,70));
		wand2.setPosition(47.5f+x,9.5f+y,0f+z);
		wand2.setMaterial(tunnel);
		tunnelstrasse[4] = wand2;
		
		Mesh dach = new Mesh(MeshFactory.createCuboid(10,75f,70));
		dach.setPosition(15f+x,34.5f+y,0f+z);
		dach.setMaterial(tunnel);
		tunnelstrasse[5] = dach;
		
		return tunnelstrasse;
	}
	
	
	public void CreateStreet(GLDrawable gld,int laenge){
        float counter = 0;
        float abstand = 0;
        float r = 0.3f;
        float g = 0.6f;
        float b = 0.75f;
        for(int i = 0; i < laenge; i++) {
	        Mesh[] strasse = Strasse(gld,false,0f,0f,counter);
	        int strasseList = DisplayListFactory.createList(gld, strasse);
	        Mesh[] strasse2 = Strasse(gld,true,0f,0f,counter-70f);
	        int tunnelstrasse = DisplayListFactory.createList(gld, strasse2);
			this.solidObjects.add(strasseList);
			this.solidObjects.add(tunnelstrasse);
			counter = counter - 140f;
        }
        
        for(int j = 0; j <laenge/2; j++) {

	    	Mesh scheibe = new Mesh(MeshFactory.createCuboid(80f,80,1f));
	    	Material glas = new Material();
	    	glas.setColor(new Color(r,g,b, 0.5f));
	        glas.setAmbientAndDiffuse(new Color(1.0f, 0.3f, 0.8f, 0.5f));
	        scheibe.setMaterial(glas);
	        scheibe.setPosition(15,0,abstand);
	        this.transparentObjects.add(DisplayListFactory.createList(gld,scheibe));
	        abstand = abstand - 280;
	        r = g/b%1.0f;
	        b = r/g%1.0f;
	        g = b/r%1.0f;
        }

	}

	
	public static int createHouseList(GL gl){
        int list = gl.glGenLists(1);
        gl.glNewList(list, GL.GL_COMPILE);
        
        //grnflche:
        gl.glColor3f(0.2f, 0.8f, 0.2f);
        
        gl.glBegin(GL.GL_QUADS);
        gl.glNormal3f(0.0f, 1.0f, 0.0f);
        gl.glVertex3f(-13.0f, 0.0f, 13.0f);
        gl.glVertex3f(13.0f, 0.0f, 13.0f);
        gl.glVertex3f(13.0f, 0.0f, -13.0f);
        gl.glVertex3f(-13.0f, 0.0f, -13.0f);
        gl.glEnd();
        
        //Hausseiten:
        gl.glColor3f(0.4f, 0.4f, 0.4f);
        
        //"vorne"
        gl.glBegin(GL.GL_QUADS);
        gl.glNormal3f(0.0f, 0.0f, 1.0f);
        gl.glVertex3f(-7.0f, 0.0f, 5.0f);
        gl.glVertex3f(5.0f, 0.0f, 5.0f);
        gl.glVertex3f(5.0f, 10.0f, 5.0f);
        gl.glVertex3f(-7.0f, 10.0f, 5.0f);
        gl.glEnd();
        
        //"rechts"
        gl.glBegin(GL.GL_QUADS);
        gl.glNormal3f(1.0f, 0.0f, 0.0f);
        gl.glVertex3f(5.0f, 0.0f, 5.0f);
        gl.glVertex3f(5.0f, 0.0f, -5.0f);
        gl.glVertex3f(5.0f, 10.0f, -5.0f);
        gl.glVertex3f(5.0f, 10.0f, 5.0f);
        gl.glEnd();
        
        // "hinten"
        gl.glBegin(GL.GL_QUADS);
        gl.glNormal3f(0.0f, 0.0f, -1.0f);
        gl.glVertex3f(5.0f, 0.0f, -5.0f);
        gl.glVertex3f(-7.0f, 0.0f, -5.0f);
        gl.glVertex3f(-7.0f, 10.0f, -5.0f);
        gl.glVertex3f(5.0f, 10.0f, -5.0f);
        gl.glEnd();
        
        // "links"
        gl.glBegin(GL.GL_QUADS);
        gl.glNormal3f(-1.0f, 0.0f, 0.0f);
        gl.glVertex3f(-7.0f, 0.0f, -5.0f);
        gl.glVertex3f(-7.0f, 0.0f, 5.0f);
        gl.glVertex3f(-7.0f, 10.0f, 5.0f);
        gl.glVertex3f(-7.0f, 10.0f, -5.0f);
        gl.glEnd();
        
        //das Dach:
        //"rechte seite"
        gl.glBegin(GL.GL_TRIANGLES);
        gl.glNormal3f(1.0f, 0.0f, 0.0f);
        gl.glVertex3f(5.0f, 10.0f, 5.0f);
        gl.glVertex3f(5.0f, 10.0f, -5.0f);
        gl.glVertex3f(5.0f, 13.0f, 0.0f);
        gl.glEnd();
        
        //"linke seite"
        gl.glBegin(GL.GL_TRIANGLES);
        gl.glNormal3f(-1.0f, 0.0f, 0.0f);
        gl.glVertex3f(-7.0f, 10.0f, 5.0f);
        gl.glVertex3f(-7.0f, 10.0f, -5.0f);
        gl.glVertex3f(-7.0f, 13.0f, 0.0f);
        gl.glEnd();
        
        gl.glColor3f(0.7f, 0.4f, 0.4f); //rote ziegel
        
        //"vorne"
        gl.glBegin(GL.GL_QUADS);
        gl.glVertex3f(-7.0f, 10.0f, 5.0f);
        gl.glVertex3f(5.0f, 10.0f, 5.0f);
        gl.glVertex3f(5.0f, 13.0f, 0.0f);
        gl.glVertex3f(-7.0f, 13.0f, 0.0f);
        gl.glEnd();
        
        //"hinten"
        gl.glBegin(GL.GL_QUADS);
        gl.glVertex3f(5.0f, 10.0f, -5.0f);
        gl.glVertex3f(-7.0f, 10.0f, -5.0f);
        gl.glVertex3f(-7.0f, 13.0f, 0.0f);
        gl.glVertex3f(5.0f, 13.0f, 0.0f);
        gl.glEnd();
        
        gl.glEndList();
        
        return list;
    }
	
	

}